[build-system]
requires = ["setuptools", "wheel"]
build-backend = "setuptools.build_meta"

[project]
name = "{{ project_name }}"
version = "0.1.0"
description = "A new Python project."
authors = [
    { name = "Your Name", email = "your.email@example.com" }
]
readme = "README.md"
license = { file = "LICENSE" }
requires-python = ">=3.11"
dependencies = []

[project.urls]
"Homepage" = "https://example.com"

[project.optional-dependencies]
dev = [
    "pytest",
    "pytest-cov",
]
